

--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("ItemBall")
M.TAG       = "ItemBall"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 配置表
local Config = require("app.numline.Config")
-- 物品骨骼的类
local ItemBone = require("app.numline.node.ItemBone")
-- 公用实物动作类
local PublicAct = require("app.numline.node.PublicAct")
-- 自动事件类
local AutoEvent = require("app.numline.node.AutoEvent")
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._main      = params.main
	self._itemId    = params.itemId
	self._scale     = params.scale
	self._touchLock = true
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
	-- 加载实物的骨骼
	self:loadItemBone()
	-- 加载自动事件
	self:loadAutoEvent()
end

-- 图片渲染
function M:loadDisplay()
	self:display("num/line/item/item"..self._itemId.."/item.png"):opacity(0)
end

-- 加载自动事件
function M:loadAutoEvent()
	local autoEvent = AutoEvent.new({
		itemId    = self._itemId,	-- 实物ID
		delayTime = 2,		        -- 自动事件的间隔时间
		itemNode  = self			-- 物品节点
	}):to(self):p(10000, 10000):unbindTouch()

	self._autoEvent = autoEvent
end

-- 加载实物的骨骼
function M:loadItemBone()
	local itemBone = ItemBone.new({
		parent       = self,
		itemId       = self._itemId,
		file         = "bone/".."qiu",
		armatureName = "qiu_piqiu",
	}):to(self, 100):p(self:cw()/2, self:ch()/2):unbindTouch()
	self._itemBone = itemBone
end

-- 点击缩放效果
function M:playClickScale()
	self:line({
		{"scaleTo", 0.05, 0.5*0.9},
		{"scaleTo", 0.15, 0.5*1.1},
		{"scaleTo", 0.1, 0.5*1},
	})
end

-- 进场效果
function M:enterEffect()
	self:activeAutoEvent()
	self:line({
		{"delay", 0.2},
		{"fn", function ()
			soundVoice:stopNowSound()
			-- 播放配音[v147194]小皮球弹呀弹
			soundVoice:playEffectv147194()
		end}
	})
end

-- 激活自动事件
function M:activeAutoEvent()
	self:line({
		{"delay", 0.2},
		A.union({
			{"fn", function ()
				-- 播放音效[sfx14712008]变形后缩小
				soundEffect:playEffectsfx14712008()
			end},
			{"scaleTo", 0.5, 0.5},
			{"moveBy", 0.5, ccp(0, -100)}
		}),
		{"fn", function ()
			-- 开启自动事件
			self._autoEvent:openAutoEvent()
			self._main:playActiveParticle()
		end}
	})
end

-- 播放打球弹起动画
function M:playBallJump(event)
	-- 播放音效[sfx14712022]皮球弹跳
	soundEffect:playEffectsfx14712022()

	self:line({
		{"fn", function ()
			self:playJumpParticle()
		end},
		self:ballJumpAct(0.6, 300, 1.5),
		{"fn", function ()
			self:playJumpParticle()
		end},
		self:ballJumpAct(0.4, 200, 1.3),
		{"fn", function ()
			self:playJumpParticle()
		end},
		self:ballJumpAct(0.3, 150, 1.15),
		{"fn", function ()
			self:playJumpParticle()
		end},
		self:ballJumpAct(0.2, 100, 1),
		{"fn", function ()
			self:playJumpParticle()
		end},
		self:ballJumpAct(0.1, 50, 0.8),
		{"fn", function ()
			self:playJumpParticle()
			if event then
				event()
			end
		end},
	})
end

-- 退场效果
function M:quitEffect()
	local callback = function ()
		self._main:clearActiveParticle()
		self._main:enterNextLevel()
	end

	self:bounceOut(self, 0, callback)
end

-- 球弹跳动作
function M:ballJumpAct(time, height, strength)
	local action = A.line({
		CUtil:jumpToLimitHeight(self, time, self:point(), height, 1, function ()
			self:feedback(self, 0.15, strength)
		end),
	})

	return action
end

-- Q弹反馈 #node:执行动作对象 #time动作总时间 #strength:强度
function M:feedback(node, time, strength, fn)
    if node._commonAction_isInFeedback then return end
    local strength = strength or 1
    node._commonAction_isInFeedback = true

    local time = time/3
    local sizeX = node:scaleX()
    local sizeY = node:scaleY()
    node:line({
        {"scaleTo", time, sizeX * (1 + 0.15 * strength), sizeY * (1 - 0.10 * strength)},
        {"scaleTo", time, sizeX * (1 + 0.10 * strength), sizeY * (1 - 0.05 * strength)},
        {"scaleTo", time, sizeX, sizeY},
        {"fn", function ()
            node._commonAction_isInFeedback = nil
            if fn then
                fn()
            end
        end}
    })
end

-- 播放跳起的粒子
function M:playJumpParticle()
	local particle = CUtil:newParticle("particle/qiu.plist"):to(self, 100):p(self._itemBone:cw()/2, 0):scale(2)
	CUtil:changeParticleColor(particle, cc.c3b(239, 86, 81))
	self:delayDeletePar(particle, 0.1, 0.1)
end

-- 延迟删除粒子
function M:delayDeletePar(particle, time1, time2)
	self:line({
		{"delay", time1},
		{"fn", function ()
			CUtil:clearParticle(particle, time2)
		end}
	})
end

-- 弹跳退场
function M:bounceOut(node, delayTime, callback)
	CUtil:changeAnchorP(node, ccp(0.5, 0))
	node:line({
		{"delay", delayTime},
		{"fn", function ()
			-- 播放音效[sfx14712019]皮球出场
			soundEffect:playEffectsfx14712019()
		end},
		A.cycle({
			A.union({
				{"jumpBy", 0.4, cc.p(200, 0), 50, 1},
				A.line({
					{"scaleTo", 0.1, node:scale()*0.9, node:scale()*1.1},
					{"scaleTo", 0.2, node:scale()*1.1, node:scale()*0.8},
					{"scaleTo", 0.1, node:scale()},
				})
			}),
			{"fn", function ()
				local particle = CUtil:newParticle("particle/qiu.plist"):to(self, 100):p(self._itemBone:cw()/2, 0):scale(2)
				CUtil:changeParticleColor(particle, cc.c3b(239, 86, 81))
				self:delayDeletePar(particle, 0.1, 0.1)
			end}
		}, 4),
		{"fn", function()
            if callback then
                callback()
            end
		end},
		{"remove"},
	})
end


----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y)
	if self._touchLock then return end
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()
	
	self:playClickScale()
	self._autoEvent:clickEvent()
	return true
end

function M:onTouchMoved(x, y)

end

function M:onTouchEnded(x, y)
	
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M