

--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("ItemCake")
M.TAG       = "ItemCake"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 配置表
local Config = require("app.numline.Config")
-- 物品骨骼的类
local ItemBone = require("app.numline.node.ItemBone")
-- 公用实物动作类
local PublicAct = require("app.numline.node.PublicAct")
-- 自动事件类
local AutoEvent = require("app.numline.node.AutoEvent")
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._main             = params.main
	self._itemId           = params.itemId
	self._scale            = params.scale
	-- 蜡烛节点集合
	self._candleTab        = {}  
	-- 烛火节点集合   
	self._fireTab 	       = {}
	-- 触控锁
	self._touchLock        = true
	-- 第几次咬蛋糕
	self._eatIndex         = 0
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
	-- 加载蛋糕实物图
	self:loadCake()
	--  加载蜡烛
	self:loadCandle()
	-- 加载烛火
	self:loadFire()
	-- 加载自动事件
	self:loadAutoEvent()
end

-- 图片渲染
function M:loadDisplay()
	self:display("num/line/item/item"..self._itemId.."/item.png"):opacity(0)
end

-- 加载自动事件
function M:loadAutoEvent()
	local autoEvent = AutoEvent.new({
		itemId    = self._itemId,	-- 实物ID
		delayTime = 2,		        -- 自动事件的间隔时间
		itemNode  = self			-- 物品节点
	}):to(self):p(10000, 10000):unbindTouch()

	self._autoEvent = autoEvent
end

-- 加载蛋糕实物图
function M:loadCake()
	local cake = D.img("num/line/item/item"..self._itemId.."/item.png"):to(self):p(self:cw()/2, self:ch()/2)
	self._cake = cake
end

-- 加载蜡烛
function M:loadCandle()
	local posTab = {ccp(107.2, 333.1), ccp(180.8, 333.1), ccp(252.1, 333.1)}
	for i = 1, 3 do
		local candle = self:createCandle(posTab[i])
		table.insert(self._candleTab, candle)
	end
end

-- 加载烛火
function M:loadFire()
	local posTab = {ccp(107.2, 387), ccp(180.8, 387), ccp(252.1, 387)}
	for i = 1, 3 do
		local fire = self:createFire(posTab[i])
		table.insert(self._fireTab, fire)
	end
end

-- 创建单个蜡烛
function M:createCandle(pos)
	local candle = D.img("num/line/item/item"..self._itemId.."/candle.png"):to(self, 1020):p(pos)
	return candle
end

-- 创建单个烛火
function M:createFire(pos)
	local fire = D.img("num/line/item/item"..self._itemId.."/fire/keep/fire1.png"):to(self, 1030):p(pos):scale(0.8):hide()
	return fire
end

-- 播放蜡烛点燃后持续燃烧帧动画方法
function M:playFireAni()
	-- 播放音效[sfx14712024]点燃蜡烛
	soundEffect:playEffectsfx14712024()

	for i, v in pairs(self._fireTab) do
		v:show()
		v:line({
			A.one({"imagerange", "num/line/item/item"..self._itemId.."/fire/out/fire", 1, 4, 0.1}):reverse(),
			{"fn", function ()
				v._cycleFireAct = A.cycle({
					{"imagerange", "num/line/item/item"..self._itemId.."/fire/keep/fire", 1, 7, 0.1},
				})
				v:runAction(v._cycleFireAct)
			end}
		})
	end
end

-- 点击缩放效果
function M:playClickScale()
	self:line({
		{"scaleTo", 0.05, 0.9},
		{"scaleTo", 0.15, 1.1},
		{"scaleTo", 0.1, 1},
	})
end

-- 进场效果
function M:enterEffect()
	self:activeAutoEvent()
	self:line({
		{"delay", 0.2},
		{"fn", function ()
			soundVoice:stopNowSound()
			-- 播放配音[v147198]生日蛋糕真美味
			soundVoice:playEffectv147198()
		end}
	})
end

-- 激活自动事件
function M:activeAutoEvent()
	self:line({
		{"delay", 0.2},
		{"fn", function ()
			-- 播放音效[sfx14712008]变形后缩小
			soundEffect:playEffectsfx14712008()
		end},
		{"moveBy", 0.5, ccp(0, -60)},
		{"fn", function ()
			self:playFireAni()
			-- 开启自动事件
			self._autoEvent:openAutoEvent()
			self._main:playActiveParticle()
		end}
	})
end

-- 播放烛火被吹灭动画
function M:playFireQuitAni(event)
	for i, v in pairs(self._fireTab) do
		v:stopAllActions()
		v:line({
			{"imagerange", "num/line/item/item"..self._itemId.."/fire/out/fire", 1, 5, 0.1},
			{"fn", function ()
				if i == 1 then
					if event then
						event()
					end
				end
			end}
		})
	end
end

-- 播放蜡烛被拔掉动画
function M:playCandleQuitAni(event)
	self._candleTab[1]:line({
		{"fn", function ()
			-- 播放音效[sfx14712025]拔掉蜡烛
			soundEffect:playEffectsfx14712025()
		end},
		A.union({
			{"jumpBy", 0.6, ccp(-300, 300), 50, 1},
			{"rotateBy", 0.5, -180}
		}),
		{"remove"}
	})
	self._candleTab[2]:line({
		{"delay", 0.3},
		{"fn", function ()
			-- 播放音效[sfx14712025]拔掉蜡烛
			soundEffect:playEffectsfx14712025()
		end},
		A.union({
			{"jumpBy", 0.6, ccp(0, 300), 50, 1},
		}),
		{"remove"}
	})
	self._candleTab[3]:line({
		{"delay", 0.6},
		{"fn", function ()
			-- 播放音效[sfx14712025]拔掉蜡烛
			soundEffect:playEffectsfx14712025()
		end},
		A.union({
			{"jumpBy", 0.6, ccp(300, 300), 50, 1},
			{"rotateBy", 0.5, 180}
		}),
		{"fn", function ()
			if event then
				event()
			end
		end},
		{"remove"}
	})
end

-- 播放蛋糕被吃一口动画（前面4口）
function M:playEatCakeAni1(event)
	self._eatIndex = self._eatIndex + 1
	self._cake:display("num/line/item/item"..self._itemId.."/item"..(self._eatIndex+1)..".png")
	-- 播放音效[sfx14712026]吃蛋糕
	soundEffect:playEffectsfx14712026()
	if event then
		event()
	end
end

-- 播放剩下的蛋糕全吃掉动画
function M:playEatCakeAni2()
	self._cake:line({
		{"delay", 0.1},
		{"fn", function ()
			self._cake:display("num/line/item/item"..self._itemId.."/item6.png")
			-- 播放音效[sfx14712026]吃蛋糕
			soundEffect:playEffectsfx14712026()
		end},
		{"delay", 0.1},
		{"fn", function ()
			self._cake:display("num/line/item/item"..self._itemId.."/item7.png")
			-- 播放音效[sfx14712026]吃蛋糕
			soundEffect:playEffectsfx14712026()
		end},
		{"delay", 0.1},
		{"fn", function ()
			self._cake:display("num/line/item/item"..self._itemId.."/item8.png")
			-- 播放音效[sfx14712026]吃蛋糕
			soundEffect:playEffectsfx14712026()
		end},
		{"delay", 0.1},
		{"fn", function ()
			self._main:clearActiveParticle()
			self._main:enterNextLevel()
		end},
		{"remove"}
	})
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()
	
	self:playClickScale()
	self._autoEvent:clickEvent()
	return true
end

function M:onTouchMoved(x, y, touches)

end

function M:onTouchEnded(x, y, touches)
	
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M