

--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("ItemTree")
M.TAG       = "ItemTree"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 配置表
local Config = require("app.numline.Config")
-- 苹果节点的类
local Apple = require("app.numline.node.Apple")
-- 公用实物动作类
local PublicAct = require("app.numline.node.PublicAct")
-- 自动事件类
local AutoEvent = require("app.numline.node.AutoEvent")

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	-- 主层级
	self._main             = params.main
	-- 物品ID
	self._itemId           = params.itemId
	-- 缩放倍数
	self._scale            = params.scale
	-- 苹果滚动的方向
	self._appleDir  	   = T.shuffle({1, -1})[1]
	-- 触控锁
	self._touchLock        = true
	-- antoEvent事件节点是否仍然执行当前fnIndex的方法
	self._isStillCurFn     = false
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	
	-- 图片渲染
	self:loadDisplay()
	-- 加载苹果树
	self:loadTree()
	-- 加载苹果
	self:loadApples()
	-- 加载自动事件
	self:loadAutoEvent()
end

-- 图片渲染
function M:loadDisplay()
	self:display("num/line/item/item"..self._itemId.."/item.png"):opacity(0)
end

-- 加载自动事件
function M:loadAutoEvent()
	local autoEvent = AutoEvent.new({
		itemId    = self._itemId,	-- 实物ID
		delayTime = 1.5,		    -- 自动事件的间隔时间
		itemNode  = self			-- 物品节点
	}):to(self):p(10000, 10000):unbindTouch()

	self._autoEvent = autoEvent
end

-- 加载苹果树
function M:loadTree()
	local tree = D.img("num/line/item/item"..self._itemId.."/item.png"):to(self):p(self:cw()/2, self:ch()/2)
	self._tree = tree
end

-- 加载苹果
function M:loadApples()
	local posTab = {
		ccp(89.9, 251.2), ccp(170.4, 210.2), ccp(243.9, 270.3),
		-- ccp(485.9+X_OFFSET, 254+Y_OFFSET), ccp(404+X_OFFSET, 295+Y_OFFSET), ccp(558+X_OFFSET, 314+Y_OFFSET),
	}
	local appleTab = {}
	for i = 1, 3 do
		local apple = self:createApple(posTab[i])
		table.insert(appleTab, apple)
	end
	self._appleTab = appleTab
end

-- 创建苹果
function M:createApple(pos)
	local this = self
	local apple = Apple.new({
		parent = self,
		main   = self._main,
	}):to(self, 1000):p(pos):scale(0):hide()
	
	return apple
end

-- 点击缩放效果
function M:playClickScale()
	self:line({
		{"scaleTo", 0.05, 0.95},
		{"scaleTo", 0.15, 1.05},
		{"scaleTo", 0.1, 1},
	})

	for i, v in pairs(self._appleTab) do
		v:line({
			{"scaleTo", 0.05, 0.8*0.95},
			{"scaleTo", 0.15, 0.8*1.05},
			{"scaleTo", 0.1, 0.8*1},
		})
	end
end

-- 进场效果
function M:enterEffect()
	self:activeAutoEvent()
	self:line({
		{"delay", 0.2},
		{"fn", function ()
			soundVoice:stopNowSound()
			-- 播放配音[v147199]苹果树香喷喷
			soundVoice:playEffectv147199()
		end}
	})
end

-- 激活自动事件
function M:activeAutoEvent()
	self:line({
		{"delay", 0.2},
		{"fn", function ()
			-- 出现苹果
			for i, v in pairs(self._appleTab) do
				v:show()
				v:getInto()
			end
			-- 播放音效[sfx14712027]苹果冒出来
			soundEffect:playEffectsfx14712027()
		end},
		{"delay", 0.5},
		{"fn", function ()
			-- 随机洗牌苹果节点顺序
			T.shuffle(self._appleTab)
			-- 开启自动事件
			self._autoEvent:openAutoEvent()
			self._main:playActiveParticle()
		end}
	})
end

-- 点击树触发的苹果掉落（非点击苹果触发）
function M:appleDropByClickTree(event)
	if #self._appleTab > 0 then
		self._appleTab[1]:appleDrop(event)
	else
		self:checkIsAppleAllDropOut(event, "tree")
	end
end

-- 检查苹果是否全部掉完 #event苹果全部掉完后的下一个事件, #clickTag 点击树或苹果的标记
function M:checkIsAppleAllDropOut(event, clickTag)
	if #self._appleTab == 0 then
		self._isStillCurFn = false
		if event then
			event()
		end
	end

	self:checkFirstClickIsTreeOrApple(clickTag)
end

-- 判断首次掉落苹果，是通过点击树还是点击苹果触发的
-- 作用：若三次都是点击苹果触发苹果掉关了, 第四次点击苹果树，会导致autoEvent还在调用fnIndex为1的苹果掉落方法，需要点第二次才退场
-- 所以首次点击苹果时直接fnIndex赋值为1，苹果全部掉落后再点击树，会调用下一个树退场的方法
function M:checkFirstClickIsTreeOrApple(clickTag)
	-- 此方法只在第一次掉落苹果时调用一次
	if not self._isFirstDrop then
		self._isFirstDrop = true
		self._isStillCurFn = true
		self._touchLock = false
		if clickTag == "apple" then
			self._autoEvent._fnIndex = 1
		end
	end
end

-- 退场效果
function M:quitEffect()
	--粒子
	local particle = CUtil:newParticle("particle/diaoyezi.plist"):to(self._main, 100):p(self:px(), self:py()+120)
	particle:cycle({
		{"delay", 1/60},
		{"fn", function ()
			if not tolua.isnull(particle) then
				particle:p(self:px(), self:py()+120)
			end
		end}
	})

	local callback = function ()
		particle:remove()
		self._main:clearActiveParticle()
		self._main:enterNextLevel()
	end
	PublicAct:bounceOut(self, 0, callback)
	-- 播放音效[sfx14712031]苹果树移出
	soundEffect:playEffectsfx14712031()
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end

----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()
	self:playClickScale()
	self._autoEvent:clickEvent(self._isStillCurFn)
	return true
end

function M:onTouchMoved(x, y, touches)

end

function M:onTouchEnded(x, y, touches)
	
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M