

--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("ItemWatermelen")
M.TAG       = "ItemWatermelen"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 配置表
local Config = require("app.numline.Config")
-- 物品骨骼的类
local ItemBone = require("app.numline.node.ItemBone")
-- 公用实物动作类
local PublicAct = require("app.numline.node.PublicAct")
-- 自动事件类
local AutoEvent = require("app.numline.node.AutoEvent")
-- 虫子吃东西时的爬行路径
local crawlPathTab = {
	cc.p(366.6, 51.5), cc.p(215.2, 63.9), cc.p(117.4, 98.3), cc.p(138, 150),
	cc.p(160.0, 200.0), cc.p(170.0, 200.0), cc.p(210, 180.0), ccp(800+X_OFFSET, 50)
}
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	self._main          = params.main
	self._itemId        = params.itemId
	self._scale         = params.scale
	-- 虫子当前爬行路径的索引
	self._curCrawlIndex = 0
	self._touchLock     = true
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 图片渲染
	self:loadDisplay()
	-- 加载实物
	self:loadItemNode()
	-- 加载毛毛虫的骨骼
	self:loadWormBone()
	-- 加载自动事件
	self:loadAutoEvent()
end

-- 图片渲染
function M:loadDisplay()
	self:display("num/line/item/item"..self._itemId.."/item.png"):opacity(0)
end

-- 加载自动事件
function M:loadAutoEvent()
	local autoEvent = AutoEvent.new({
		itemId = self._itemId,	-- 实物ID
		delayTime = 2,			-- 自动事件的间隔时间
		itemNode = self			-- 物品节点
	}):to(self):p(10000, 10000):unbindTouch()

	self._autoEvent = autoEvent
end

-- 加载实物
function M:loadItemNode()
	local itemNode = D.img("num/line/item/item"..self._itemId.."/item.png"):to(self):p(self:cw()/2, self:ch()/2)
	self._itemNode = itemNode
end

-- 加载毛毛虫的骨骼
function M:loadWormBone()
	local wormBone = ItemBone.new({
		parent       = self,
		itemId       = self._itemId,
		file         = "bone/".."mmc",
		armatureName = "chon",
	}):to(self, 100):p(800+X_OFFSET, 50):rotate(90):unbindTouch():hide()
	self._wormBone = wormBone
end

-- 点击缩放效果
function M:playClickScale()
	self:line({
		{"scaleTo", 0.05, 0.9},
		{"scaleTo", 0.15, 1.1},
		{"scaleTo", 0.1, 1},
	})
end

-- 进场效果
function M:enterEffect()
	local time = sound:soundDuration("v147192")
	self:line({
		{"delay", 0.2},
		{"fn", function ()
			soundVoice:stopNowSound()
			-- 播放配音[v147192]两口吃掉小西瓜
			soundVoice:playEffectv147192()
		end},
		{"delay", time},
		{"fn", function ()
			self:activeAutoEvent()
		end}
	})
end

-- 激活自动事件
function M:activeAutoEvent()
	-- 开启自动事件
	self._autoEvent:openAutoEvent()
	self._main:playActiveParticle()
end

-- 通过两点求虫子的角度
function M:getAngleByPos(startPos, endPos)
	local x = startPos.x - endPos.x
	local y = startPos.y  - endPos.y 
	local angle = math.atan2(y, x)*180/math.pi
	return angle 
end

-- 毛毛虫爬着吃西瓜的方法 #delayTime允许点击的延迟时间, #event点击触发的下一个事件
function M:wormEatWatermelen(delayTime, event)
	self._curCrawlIndex = self._curCrawlIndex + 1
	local startPos = self._wormBone:point()
	local endPos = crawlPathTab[self._curCrawlIndex]
	local dist = ccpDistance(startPos, endPos)

	self._wormBone:playCrawl()
	local time = dist/300
	local angle = self:getAngleByPos(startPos, endPos)

	self._wormBone:rotation(90 - angle)
	self._wormBone:line({
		{"moveTo", time, endPos},
		{"fn", function ()
			self._wormBone:playEatAni()
			self._itemNode:display("num/line/item/item"..self._itemId.."/"..self._curCrawlIndex..".png")
		end},
		{"delay", delayTime},
		{"fn", function ()
			if event then
				event()
			end
		end}
	})
end

-- 毛毛虫在屏幕外面爬进来吃
function M:playWormEnter(event)
	self._wormBone:show()
	self._wormBone:playCrawlEnter()
	self._wormBone:line({
		{"moveTo", 1, ccp(400.0, 50)},
		{"fn", function ()
			self:wormEatWatermelen(1, event)
		end}
	})
end

-- 播放虫子爬行吃西瓜
function M:playEatWatermelen(event)
	self:wormEatWatermelen(0, event)
end

-- 退场效果
function M:quitEffect()
	self._curCrawlIndex = self._curCrawlIndex + 1
	local startPos = self._wormBone:point()
	local endPos = crawlPathTab[self._curCrawlIndex]
	local dist = ccpDistance(startPos, endPos)

	self._wormBone:playCrawlOut()
	local time = 1 --dist/100
	local angle = self:getAngleByPos(startPos, endPos)

	self._wormBone:rotation(90 - angle)
	self._wormBone:line({
		{"moveTo", time, endPos},
		{"fn", function ()
			self._wormBone:stopCrawlSoundEffect()
			self._main:clearActiveParticle()
			self._main:enterNextLevel()
		end}
	})
end
----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------
function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()
	
	self:playClickScale()
	self._autoEvent:clickEvent()
	return true
end

function M:onTouchMoved(x, y, touches)

end

function M:onTouchEnded(x, y, touches)
	
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M