--[[

Copyright (c) 2012-2020 Baby-Bus.com
Author: cxy
Date: 2021-2-7

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 数字基类
----------------------
local M     = classSpriteTouch("NumBase")
M.TAG       = "NumBase"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
local Config 			= require("app.numwrite.Config")

local CalculatePoint 	= require("app.numwrite.CalculatePoint")



----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	-- 层
	self._layer 		= params.layer
	-- 箭头写字完成回调
	self._arrowCallback = params.arrowCallback or nil
	-- 是否可以播放拖动时的声音
	self._canPlayEffect = false
end




----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)
	-- 加载渲染以外的其他操作
	self:loadRenderOtherwise()
	-- 图片渲染
	self:loadDisplay()

end

function M:loadRenderOtherwise()
	-- 轨迹路径，分段
	self._pathPoints 	= CalculatePoint["calculatePoint"..self._number](CalculatePoint)
	-- 将所有轨迹路径合并成一个表
	self._allPathPts 	= CalculatePoint:mergeVectorTable(self._pathPoints)
	-- 笔头的纹理
	self._strokeTex 	= R.loadTexture("num/write/stokehead.png")
end

-- 图片渲染
function M:loadDisplay()
	self:setTexture("num/write/num"..self._number.."/base.png"):opacity(0)
	-- 画出轮廓
	self:drawAllStroke()

	-- 获取画板的纹理信息做clippingNode
	local img = self._renderSprite:getTexture()
	-- D.img(img):to(self, 999):p(self:cw()/2 + 300, self:ch()/2):flipY(true)
	-- 裁切
	local clipImg	= D.img(img):flipY(true):p(self:cw()/2, self:ch()/2)
	-- 创建遮罩
	local clip 		= CCNodeExtend.extend(cc.ClippingNode:create(clipImg))
	self:addChild(clip, 1)
	-- 设置裁剪区域的显示，false为显示裁剪的区域，true为裁剪后的区域(很关键)
	clip:setInverted(false)
	-- 设置alpha阈值， 只有模板（stencil）的alpha像素大于alpha阈值（alphaThreshold）时内容才会被绘制。
	-- alpha阈值（threshold）范围应是0到1之间的浮点数。 alpha阈值（threshold）默认为1。
	clip:setAlphaThreshold(0)
	self._clip 		= clip
end


-- 自动写出笔迹
function M:loadAutoStroke()
	local strokes 	= {} 					-- 全部笔迹
	local posNum 	= #self._allPathPts 	-- 点的数量
	local posIndex 	= 0 					-- 当前最前面的点的索引
	local speed 	= 300	 				-- 写字速度
	local frameStep = speed / 60 			-- 每帧跳过的点的数量
	local frameNum 	= posNum / frameStep 	-- 共有多少帧
	local stokeSize = 1 					-- 笔画粗细
	local scale 	= 120 / 120
	local color 	= self:getBrushColor()
	-- 循环调度将笔迹画出
	self:line({
		A.cycle({
			{"delay", 1/60},
			{"fn", function()
				-- 每帧画出沿途所有的点
				local preIndex 	= math.floor(posIndex)
				posIndex 		= posIndex + frameStep
				for i = preIndex, math.floor(posIndex) do
					if self._allPathPts[i] then
						local renderNode = D.img(self._strokeTex):p(self._allPathPts[i])
						renderNode:to(self, 1):color(color):scale(scale)
						table.insert(strokes, renderNode)
					end
				end
			end},
		}, frameNum),
		{"delay", 1/60},
		{"fn", function()
			self:drawAllStroke(self:getBrushColor())
			self:clearNodeTable(strokes)
			self._layer:playNumberVoice()
		end},
		{"delay", 0.05},
		{"fn", function()
			self:loadWriteItem()
			self:loadFullMaskGuide()
		end},
	})
end

-- 显示完整的字(每一个点画上) #笔迹颜色
function M:drawAllStroke(color)
	-- 首次创建画布
	if not self._renderTexture then
		local cw, ch 		= self:cw(), self:ch()
		local render 		= CCNodeExtend.extend(cc.RenderTexture:create(cw + 32, ch + 32)):to(self):p(cw/2, ch/2)
		self._renderTexture = render
		self._renderSprite 	= CCNodeExtend.extend(render:getSprite())
	end

	local scale 	= 120 / 120
	self._renderTexture:begin()
	for i,v in ipairs(self._allPathPts) do
		local node  = D.img(self._strokeTex):p(v.x + 16, v.y + 16):setScale(scale)
		if color then node:setColor(color) end
		node:visit()
	end
	self._renderTexture:endToLua()
end


-- 加载形似书写
function M:loadWriteItem()
	print("未被重写")
end


-- 书写箭头，用作结束规范书写
function M:loadWriteArrow()
	-- 箭头坐标索引
	self._arrowPtIndex 	= 1
	local arrow = require("app.numwrite.node.Arrow").new({
		layer 		= self._layer,
		numNode 	= self
	}):to(self, 200)
	self._arrow = arrow

	arrow:scheduleUpdateWithPriorityLua(function ()
		self:updateArrowInfo()
	end, 10)

	self._arrowStokes = {}
	local ptNum 	= #self._allPathPts
	local color 	= self:getBrushColor(2)
	local size 		= 1.05
	local texture 	= self._strokeTex
	for i=1, ptNum do
		local dot 	= D.img(texture):to(self, 199):color(color):scale(size):p(self._allPathPts[i]):hide()
		table.insert(self._arrowStokes, dot)
	end
	self._arrowInterval 	= interval

	self:openGuide("_arrowPtIndex", nil, nil, function ()
		soundEffect:playEffectsfx14711017()
	end)
end

-- 箭头初始角度
function M:updateArrowInfo()
	local p1, p2
	if self._allPathPts[self._arrowPtIndex + 1] then
		p1 	= self._allPathPts[self._arrowPtIndex]
		p2 	= self._allPathPts[self._arrowPtIndex + 1]
	else
		p1 	= self._allPathPts[self._arrowPtIndex - 1]
		p2 	= self._allPathPts[self._arrowPtIndex]
	end

	self._arrow:p(self._allPathPts[self._arrowPtIndex])
	if not (p1.x == p2.x and p1.y == p2.y) then
		self._arrow:rotate(-PT.quadAngle(p1, p2) - 90)
	end

	local stokeNum 	= #self._arrowStokes
	for i,v in ipairs(self._arrowStokes) do
		if i <= self._arrowPtIndex then
			v:show()
		else
			v:hide()
		end
	end
end

-- 箭头触控移动方法
function M:arrowTouchMove(x, y)
	local newIndex, minDis = self:getNearestPtIndex(self._arrowPtIndex, x, y)

	if not (self._breakIndex and self._writedOne and self._arrowPtIndex <= self._breakIndex) then
		if newIndex > self._arrowPtIndex then
			self._arrowPtIndex = newIndex
		end
	end

	if self._arrowPtIndex >= #self._allPathPts * 0.9 then
		self:arrowWriteFinish()
		return SIGN_TOUCH_MOVED_STOP
	elseif self._breakIndex and self._arrowPtIndex == self._breakIndex then
		self._writedOne = true
		self._arrowPtIndex = self._breakIndex + 1
		self._arrow._touchLock = false
		return SIGN_TOUCH_MOVED_STOP
	elseif minDis > 80 then
		self:arrowBack()
		return SIGN_TOUCH_MOVED_STOP
	end
end

-- 箭头退回
function M:arrowBack()
	-- local endValue = (self._breakIndex and self._writedOne) and (self._breakIndex + 1) or 1
 --    self:indexMoveTo("_arrowPtIndex", endValue, nil, true, function ()
        self._arrow._touchLock = false
        self._arrow:scale(1.0)
        self._arrow:stopPullEffect()
		self:openGuide("_arrowPtIndex", nil, nil, function ()
			soundEffect:playEffectsfx14711017()
		end)
	-- end)
end


-- 拖动箭头标准写法完成
function M:arrowWriteFinish()
	self:stopGuide()
    if self._arrowFinish then return end
    self._arrowFinish = true
    self:indexMoveTo("_arrowPtIndex", #self._allPathPts, nil, true, function ()
    	self._layer:playNumberVoice()
	    self._arrow:unscheduleUpdate()
	    self._arrow:unbindTouch()
	    self:drawAllStroke(self:getBrushColor(2))
	    self:clearNodeTable(self._arrowStokes)

	    self._arrow:line({
	    	{"scaleTo", 0.5, 0},
	    	{"remove"},
	    })

	    self._renderSprite:line({
	    	{"fadeTo", 0.5, 0},
	    })

	    if self._arrowCallback then
	    	self:line({
	    		{"delay", 0.5},
	    		{"fn", function()
	    			self._arrowCallback()
	    		end},
	    		{"remove"},
	    	})
	    end
    end)
end

-- 获取距离最近的点索引，#当前索引，#x，#y
-- 返回值:最近的点索引，最近的距离
function M:getNearestPtIndex(nowIndex, x, y)
	local lp 	= self:convertToNodeSpace(cc.p(x, y))
    local began = nowIndex - 0
    local ended = nowIndex + 80
    began       = began < 1 and 1 or began
    ended       = ended > #self._allPathPts and #self._allPathPts or ended

    local minIndex      = began
    local minDistance   = 0xFFFF
    for i = began, ended do
        local tp    = self._allPathPts[i]
        local dis   = (tp.x - lp.x) * (tp.x - lp.x) + (tp.y - lp.y) * (tp.y - lp.y)
        if dis < minDistance then
            minIndex    = i
            minDistance = dis
        end
    end
   return minIndex, math.sqrt(minDistance)
end


function M:getMinIndex(pos)
    if not pos then return end
    local minDis = 0xFFFF
    local index = 1
    for i,v in ipairs(self._allPathPts) do
        local tp    = self._allPathPts[i]
        local dis   = (tp.x - pos.x) * (tp.x - pos.x) + (tp.y - pos.y) * (tp.y - pos.y)
        if dis < minDis then
            index    = i
            minDis = dis
        end
    end
    return index
end



-- 索引退回 #索引值的键（自身的成员值），#结束值，#回调
function M:indexMoveTo(keyName, endValue, speed, isEasing, callback)
	if not self[keyName] then
		print("没有这个键")
	end

	local time 		= math.abs(self[keyName] - endValue) / (speed or 600)

	local moveAct 	= isEasing and {"easing", "SINEIN", {"moveTo", time, cc.p(endValue, 0)}}
		or {"moveTo", time, cc.p(endValue, 0)}

    -- 用一个动作系统模拟回滚运动（使用缓动），动作获取执行动作的精灵的坐标来设置坐标索引
    local actNode 	= U.loadNode({}):to(self):p(self[keyName], 0)
    actNode:line({
        moveAct,
        {"fn", function()
        	actNode:unscheduleUpdate()
            self[keyName] = endValue
        end},
        {"delay", 1/60},
        {"fn", function()
           	if callback then
           		callback()
           	end
        end},
        {"remove"},
    })
    -- 用调度器更新索引值
    actNode:scheduleUpdateWithPriorityLua(function ()
    	local value 	= math.floor(actNode:px() + 0.5)
        self[keyName]   = value < 1 and 1 or value
    end, 1)

    return actNode
end


-- 开启引导 #索引键值
function M:openGuide(keyName, dis, speed, callback)
	self:loadFullMaskGuide()
    if not self._guideAct then
    	self._preGuideIndex = self[keyName]
    	self._nowGuideKey 	= keyName
    	
    	dis 		= dis or 30
    	speed 		= speed or 150
    	local time 	= dis / speed
        local action = A.cycle({
            {"delay", 3},
            {"fn", function()
            	self:loadGuideFinger(keyName, dis)
            	if callback then
            		callback()
        		else
        			soundEffect:playEffectsfx14700011()
        		end
            	-- 播放配音[v147169]写一写
            	soundVoice:playEffectv147169()
            	self._guideNode = self:indexMoveTo(keyName, self._preGuideIndex + dis, speed, true)
            end},
            {"delay", time},
            {"fn", function()
            	self._guideNode = self:indexMoveTo(keyName, self._preGuideIndex, speed, true) 
            end},
            {"delay", time},
            {"fn", function()
            	self._guideNode = self:indexMoveTo(keyName, self._preGuideIndex + dis, speed, true)
            end},
            {"delay", time},
            {"fn", function()
            	self._guideNode = self:indexMoveTo(keyName, self._preGuideIndex, speed, true) 
            end},
            {"delay", time},
        })
        self._guideAct  = action
        self:runAction(self._guideAct)
    end
end

-- 关闭引导
function M:stopGuide()
	self:stopFullMaskGuide()
    if self._guideAct then
        self:stopAction(self._guideAct)
        self._guideAct = nil
        if not tolua.isnull(self._guideNode) then
        	self._guideNode:remove()
        end
        self[self._nowGuideKey] = self._preGuideIndex

	    if self._finger then
	        self._finger:closeHandGuide()
	        self._finger:remove()
	        self._finger = nil
	    end
    end
end

-- 
function M:loadGuideFinger(keyName, dis)
	local starPos = self._allPathPts[self[keyName]]
	local endPos  = self._allPathPts[self[keyName] + 100] or self._allPathPts[#self._allPathPts]

    local finger    = require("app.shapefeed.node.GuideFinger").new({
        startPos    = starPos,
        endPos      = endPos,
    }):to(self, 1200):p(0, 0)
    self._finger = finger
    self._finger:openHandGuide(nil, nil, nil, 1)
    self._finger:line({
    	{"delay", 3},
    	{"fn", function()
    		if self._finger then
		        self._finger:closeHandGuide()
		        self._finger:remove()
		        self._finger = nil
		    end
    	end},
    })
end


-- 笔刷颜色
function M:getBrushColor(index)
	return Config._strokeColor[self._number][index or 1]
end

-- 清除表中的精灵
function M:clearNodeTable(tab)
	for i=#tab, 1, -1 do
		tab[i]:remove()
		table.remove(tab, i)
	end
end

-- 全屏引导遮罩
function M:loadFullMaskGuide()
	if self._fullMask then return end
	if not self._loadMaskAct then
		self._loadMaskAct = A.line({
			{"delay", 5},
			{"fn", function()
				self._loadMaskAct = nil
				local mask = U.loadNodeMask({
					contentSize = CCSize(1500, 1300),
					color = COLOR3_BLACK,
					opacity     = 0,
				}):to(self, - 1000):p(self:cw()/2, self:ch()/2):unbindTouch()
				mask:line({
					{"fadeTo", 0.3, 180},
				})

				self._fullMask = mask
			end},
		}):at(self)
	end
end

function M:stopFullMaskGuide()
	if self._loadMaskAct then
		self:stopAction(self._loadMaskAct)
		self._loadMaskAct = nil
	end

	if self._fullMask then
		self._fullMask:stopAllActions():line({
			{"fadeTo", 0.3, 0},
			{"hide"},
		})
	end	
end


--------------------------
-- 触控
-------------------------- 








----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end

return M