--[[Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]
----------------------
-- 类
----------------------
local M    = classSpriteTouch("TipNode")
M.TAG    = "TipNode"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
local DefaultColor = cc.c3b(255, 255, 102)
local ColorTab = {
    cc.c3b(243, 80, 122), cc.c3b(69, 227, 255), cc.c3b(199, 240, 102),
    cc.c3b(255, 212, 40), cc.c3b(255, 206, 9), cc.c3b(246, 73, 81),
}
local MoveClass = require("app.shapeline.node.MoveClass")
----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
    -- [超类调用]
    M.super.ctor(self, params)

    self._main    = params.main
    self._shapeId = params.shapeId
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
    -- [超类调用]
    M.super.onRender(self)
    -- 图片渲染
    self:loadDisplay()
    -- 加载光环圈圈
    self:loadCircle()
end

-- 图片渲染
function M:loadDisplay()
    self:display("shape/line/outline/tip.png"):scale(0):color(DefaultColor)
    self._touchLock = true
end

-- 加载光环圈圈
function M:loadCircle()
    self._circle = D.img("shape/line/outline/circle.png"):to(self):p(self:cw()/2, self:ch()/2):color(DefaultColor):scale(0.7)
end

-- 根据形状颜色ID修改提示点的颜色
function M:changeColor()
    self:color(ColorTab[self._shapeId])
    self._circle:color(ColorTab[self._shapeId])
end

-- 入场效果
function M:enterEffect()
    -- 播放音效[sfx14706007]小圆点冒出/小圆点冒出的时候播放
    soundEffect:playEffectsfx14706007()
    self:line({
        {"easing", "backOut", {"scaleTo", 0.5, 1}},
        {"fn", function ()
            self:openGuide()
            self._touchLock = false
        end}
    })
end

-- 退场效果
function M:quitEffect()
    self:remove()
end

-- 停止提示点击的效果
function M:stopTipClick()
    self._main._clickIndex = self._main._clickIndex + 1
    
    -- 播放音效/连点
	local effectIndex = totable({
		"sfx14706001", "sfx14706002", "sfx14706003", "sfx14706004", "sfx14706005",
		"sfx14706006", "sfx14706031",
	})[self._main._clickIndex]
	soundEffect["playEffect"..effectIndex](soundEffect)

    self:stopGuide()
    self._circle:stopAction(self._tipClickAct)
    self._tipClickAct = nil
    self._circle:hide()

    -- 新版点击效果
    self:line({
        {"scaleTo", 0.1, 0.5},
        {"scaleTo", 0.2, 1},
        {"scaleTo", 0.1, 0.7},
    })
end

-- 开启缩放引导
function M:openGuide()
	if self._isOpenGuide then return end
	self._isOpenGuide = true
	self:openScaleGuide(self._circle, 0.7, 1/0.7, 4, function ()
		self:playGuideVoice()
        self._main:playFingerClickGuide(self)
	end)
end

-- 播放引导台词
function M:playGuideVoice()
	soundVoice:stopNowSound()
	-- 播放配音[v147070]点一点
	soundVoice:playEffectv147070()
end

-- 关闭缩放引导
function M:stopGuide()
	if self._isOpenGuide then
		self._isOpenGuide = false
		self:stopScaleGuide(self._circle)
        self._main:stopFingerGuide()
	end
end

-- 开启缩放引导 #精灵，缩放倍数
function M:openScaleGuide(node, scaleSize, scaleMultiple, time, callback)
    if not node._commonAction_guideAct then
        scaleMultiple   = scaleMultiple or 1.2
        local action    = A.cycle({
            {"delay", time or 3},
            {"fn", function()
                soundEffect:playEffectsfx14700011()
                if callback then
                    callback()
                end
            end},
            {"scaleto", 0.2, scaleSize * scaleMultiple},
            {"scaleto", 0.1, scaleSize},
            {"scaleto", 0.2, scaleSize * scaleMultiple},
            {"scaleto", 0.1, scaleSize},
        })
        node._commonAction_guideAct = action
        node._commonAction_iniScale = scaleSize
        node:runAction(node._commonAction_guideAct)
    end
end

-- 关闭缩放引导
function M:stopScaleGuide(node)
    if node._commonAction_guideAct then
        node:stopAction(node._commonAction_guideAct)
        node._commonAction_guideAct = nil
        node:scale(node._commonAction_iniScale)
    end
end

--------------------------
-- 触控
-------------------------- 
function M:onTouchBegan(x, y, touches)
    if self._touchLock then return end
    soundVoice:stopNowSound()
    self:changeColor()
    
    -- 粒子
    local particle = CUtil:newParticle("particle/twdj.plist"):to(self, 100):p(self:cw()/2, self:ch()/2)
    self._main._menu:barChange(false)
    self._main:tipAndTargetTouchLock(true)
    self._main._targetNode:moveToNextTip(self._main._tipIndex)

    self:stopTipClick()
    --  触控有效
    return SIGN_TOUCH_BEGAN_SWALLOWS
end

function M:onTouchMoved(x, y, touches)
    return true
end

function M:onTouchEnded(x, y, touches)
    return true
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
    -- [超类调用]
    M.super.onDestructor(self)
end

----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)

end


return M