--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]

--[[!--

场景层类，定义层相关操作方法及逻辑实现。

-   定义场景层功能方法。

]]

----------------------
-- 类
----------------------
local M = classLayerTouch("Main")


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
local Config = require("app.shapepuzzle.Config")                     -- 配置表
local CameraNode = require("app.shapepuzzle.node.CameraNode")        -- 模拟镜头效果的节点
local Tray = require("app.shapepuzzle.node.Tray")          			 -- 托盘
local Shape = require("app.shapepuzzle.node.Shape")                  -- 形状道具
local GuideFinger = require("app.common.GuideFinger")   			 -- 手指引导
-- [操作变量]
-- ..

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义视图实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	-- 拼图形状出现的ID顺序队列
	self._shapeIdTab = {}
	-- 当前是第几个形状拼图       
	self._curShapeIndex = 0
	-- 全局触控锁，用于强引导时
	self._globalTouchLock = true
end

----------------------
-- 结点渲染
----------------------
--[[--

视图渲染，处理视图结点加载、事件绑定等相关操作
]]

function M:onRender()

	-- 加载背景图
	self:loadBg()
	-- 获取当前轮次的关卡组合表
	self:getCurLevelTab()
	-- 加载手指引导
	self:loadGuideFinger()
	-- 加载托盘
	self:loadTray()
	-- 加载镜头节点
	self:loadCameraNode()
	-- 加载关卡
	self:loadLevel()
	-- 播放初始台词
	self:playStartVoice()
end

function M:onEnterTransitionFinish()
    -- 播放背景音乐[bgm14704]
    sound:playbgm14704()
end

-- 加载背景图
function M:loadBg()
	local bg = U.loadBg("shape/puzzle/bg.png"):to(self)
end

-- 获取当前轮次的关卡组合表
function M:getCurLevelTab()
	if (not G_shapepuzzle_turnIndex) or G_shapepuzzle_turnIndex == 0 or G_shapepuzzle_turnIndex >= #G_shapepuzzleLevelTab then
		G_shapepuzzle_turnIndex = 1
	else
		G_shapepuzzle_turnIndex = G_shapepuzzle_turnIndex + 1
	end
	-- 关卡组合表
	self._levelTab = G_shapepuzzleLevelTab[G_shapepuzzle_turnIndex]
	-- 关卡索引
	self._levelIndex = 1
	-- 初始化关卡ID
	self._curLevel = self._levelTab[self._levelIndex]
end

-- 加载托盘
function M:loadTray()
	local tray = Tray.new({
		main = self
	}):to(self, 100):p(880+X_OFFSET*2 + 400, 120)

	self._tray = tray
end

-- 加载镜头节点
function M:loadCameraNode()
	local camera = CameraNode.new({
		main = self
	}):to(self):setContentSize(V.w, V.h):p(0, 0):anchor(ccp(0, 0))
	self._camera = camera
end

-- 加载关卡
function M:loadLevel()
	DragonBone.removeAll()
	self._shapeIdTab = Config.objectInfo[self._curLevel].shapeIdTab
	self._shapeTagTab = Config.objectInfo[self._curLevel].shapeTagTab
	self._camera._shapeTab = {}
	self._curShapeIndex = 0
	self:enterFirstShape()
end

-- 进入下一关
function M:enterNextLevel()
	if self._levelIndex >= 3 then
		self:enterNextScene()
		return
	end
	self._levelIndex = self._levelIndex + 1
	self._curLevel = self._levelTab[self._levelIndex]
	self:loadLevel()
end

-- 	播放初始台词
function M:playStartVoice()
	-- 播放配音[v147121]拼一拼~看它会变成什么呢？
	soundVoice:playEffectv147121()
end

-- 进入下一个场景
function M:enterNextScene()
    local encourage = CUtil:loadEncourage(self, 4, function ()
    	G_addFinish("shapepuzzle")
        SceneController:enterNextMoudleScene()
    end)
end

-- 第一个拼图形状道具进场
function M:enterFirstShape()
	self._curShapeIndex = self._curShapeIndex + 1
	local shapeTag = self._shapeTagTab[self._curShapeIndex]
	local shapeId = self._shapeIdTab[self._curShapeIndex]
	self._camera:enterNextOutline(self._curLevel, shapeId, shapeTag, self._curShapeIndex)

	local imgPath = "shape/puzzle/object"..self._curLevel.."/shape/shape"..shapeId..".png"
	local pos = ccp(self._tray:px()-47, self._tray:py())
	local scale = self:getShapeScale()
	local shape = self:createShape(shapeTag, imgPath, pos, scale)
	self._curShape = shape

	-- 	首关第一个形状延时后做强引导（不和初始台词冲突）
	if self._levelIndex == 1 then
		self._tray:getInto(shape, function ()
			self:line({
				{"delay", 1},
				{"fn", function ()
					self:playForceFingerMoveGuide(function ()
						self._curShape:checkIsNearOutline()
					end)
				end}
			})
		end)
	else
		self._globalTouchLock = false
		self._tray:getInto(shape, function ()
			self._curShape:openGuide()
		end)
	end
end

-- 托盘收起后，下一个拼图形状道具进场
function M:enterNextShape()
	if self._curShapeIndex >= #self._shapeIdTab then
		print("完成当前场景全部拼图块")
		self:onPuzzleAllShapeHandler()
		return
	end

	local callback = function ()
		self._curShapeIndex = self._curShapeIndex + 1
		local shapeTag = self._shapeTagTab[self._curShapeIndex]
		local shapeId = self._shapeIdTab[self._curShapeIndex]
		self._camera:enterNextOutline(self._curLevel, shapeId, shapeTag, self._curShapeIndex)

		local imgPath = "shape/puzzle/object"..self._curLevel.."/shape/shape"..shapeId..".png"
		local pos = ccp(self._tray:px()-47, self._tray:py())
		local scale = self:getShapeScale()
		local shape = self:createShape(shapeTag, imgPath, pos, scale)
		self._curShape = shape
		self._tray:getInto(shape, function ()
			shape:openGuide()
		end)
	end

	self._tray:getOut(callback)
end

-- 获取形状缩放倍数
function M:getShapeScale()
	local scale = Config.objectInfo[self._curLevel].cameraScaleTab[self._curShapeIndex]
	return scale
end

-- 创建拼图形状道具
function M:createShape(shapeTag, imgPath, pos, scale)
	local shape  = Shape.new({
		main     = self,
		shapeTag = shapeTag,
		imgPath  = imgPath,
		initPos  = ccp(pos.x-400, pos.y),
		scale    = scale
	}):to(self, 101):p(pos):scale(scale)

	-- 拼车子去广告
	if self._curLevel == 3 and self._curShapeIndex == 7 then
		CUtil:removeAd()
	-- 拼乌龟去广告
	elseif self._curLevel == 4 and self._curShapeIndex == 6 then
		CUtil:removeAd()
	end

	return shape
end

-- 获取形状到虚线框的视觉距离
function M:getShapeToOutlineDist()
	local pos1 = self._camera._curOutline:worldpoint()
	local pos2 = self._curShape:worldpoint()
	return ccpDistance(pos1, pos2)
end

-- 当拼图块全部完成时的回调
function M:onPuzzleAllShapeHandler()
	self._tray:getOut()
	
	self._camera:cameraReturn()
end

-- 播放激励粒子
function M:playActiveParticle()
	-- 粒子（播放物品激励动画时）
	local particle = CUtil:newParticle("particle/ptjl.plist"):to(self, 1000):p(V.w_2, V.h_2)
	self._activeParticle = particle
end

-- 清除激励粒子
function M:clearActiveParticle()
	if not tolua.isnull(self._activeParticle) then
		CUtil:clearParticle(self._activeParticle, 1)
	end
end

-- 加载手指引导
function M:loadGuideFinger(pos1, pos2)
    if not self._finger then
        local finger    = GuideFinger.new({
            startPos    = pos1,
            endPos      = pos2,
        }):to(self, 10000):p(0, 0)
        self._finger = finger
    end
end

-- 播放首次手指拖动强引导
function M:playForceFingerMoveGuide(callback)
	local startPos = self._curShape:point()
	local endPos = self._camera._curOutline:worldpoint()
	endPos.x = endPos.x + 20
	endPos.y = endPos.y + 20
	self._finger:playForceFingerMoveGuide(self._curShape, startPos, endPos, callback)
end

-- 播放手指拖动引导（跟随缩放引导的line动作，非cycle）
function M:playFingerMoveGuide()
	local startPos = self._curShape:point()
	local endPos = self._camera._curOutline:worldpoint()
	self._finger:playFingerMoveGuide(startPos, endPos)
end

-- 停止手指引导(拖动/点击通用)
function M:stopFingerGuide()
	self._finger:closeHandGuide()
end

----------------------
-- 结点析构
----------------------
--[[--

视图析构，处理视图结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end

function M:onExitTransitionStart()
	
end

return M