

--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("AutoEvent")
M.TAG       = "AutoEvent"

----------------------
-- 公共参数
----------------------
-- [常量]
-- ..

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	-- 实物ID
	self._itemId    = params.itemId
	-- 物品节点
	self._itemNode  = params.itemNode
	-- 自动事件的间隔时间
	self._delayTime = 10000 or params.delayTime
	-- 方法索引
	self._fnIndex   = 0
	-- 当前的方法
	self._curFn     = nil
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)

	-- 初始化数据
	self:initData(self._itemNode)
end

-- 初始化数据
function M:initData(node)
	-- 物品节点的动作方法集合
	self._actFnTab = {
		{node.playEatAni1, node.playEatAni1, node.playEatAni1, node.playEatAni1, node.playEatAni2},
		{node.playLamplineAni, node.playLampGetInto},
		{node.quitEffect},
		{node.playHideHead, node.playTortoiseCrawl1, node.playTortoiseCrawl2},
		{node.playFlagAni, node.playQueenAni, node.quitEffect},
		{node.playFireQuitAni, node.playCandleQuitAni, node.playEatCakeAni1, node.playEatCakeAni1, node.playEatCakeAni1, node.playEatCakeAni1, node.playEatCakeAni2},
	}
end

-- 物品节点的点击事件
function M:clickEvent()
	self:stopAutoEvent()
	self:runNextAct()
end
-- 执行下一个动作
function M:runNextAct()
	-- 执行动作时，物品节点触控锁定
	self._itemNode._touchLock = true

	-- 最后一个动作结束后，不再继续
	if self._fnIndex >= #self._actFnTab[self._itemId] then return end

	self._fnIndex = self._fnIndex + 1
	self._curFn = self._actFnTab[self._itemId][self._fnIndex]
	-- 执行完动作后，传一个自动事件的回调
	self._curFn(self._itemNode, function ()
		self:openAutoEvent()
	end)
end

-- 开启自动事件（延时自动执行下一个动作（时间范围内支持点击触发执行下一个动作），不支持重复当前动作）
function M:openAutoEvent()
	self._itemNode._touchLock = false
	self:openGuide()
	self._autoAct = A.line({
		{"delay", self._delayTime},
		{"fn", function ()
			self:runNextAct()
		end}
	}):at(self)
end

-- 关闭自动事件
function M:stopAutoEvent()
	self:stopGuide()
	if self._autoAct then
		self:stopAction(self._autoAct)
		self._autoAct = nil
	end
end

---------------------------------------引导----------------------------------------------
-- 开启引导
function M:openGuide()
	if self._isOpenGuide then return end
	self._isOpenGuide = true
	local scale = (self._itemNode:getScaleX() + self._itemNode:getScaleY())/2
	self:openScaleGuide(self._itemNode, scale, 1.1, 3, function ()
		self:playGuideVoice()
		self:playFingerClickGuide()
	end)
end

-- 播放引导台词
function M:playGuideVoice()
	soundVoice:stopNowSound()
	-- 播放配音[v147070]点一点
	soundVoice:playEffectv147070()
end

-- 关闭引导
function M:stopGuide()
	if self._isOpenGuide then
		self._isOpenGuide = false
		self:stopScaleGuide(self._itemNode)
		self:stopFingerGuide()
	end
end

-- 开启缩放引导 #精灵，缩放倍数
function M:openScaleGuide(node, scaleSize, scaleMultiple, time, callback)
    if not node._commonAction_guideAct then
        scaleMultiple   = scaleMultiple or 1.2
        local action    = A.cycle({
            {"delay", time or 3},
            {"fn", function()
                soundEffect:playEffectsfx14700011()
                if callback then
                    callback()
                end
            end},
            {"scaleto", 0.2, scaleSize * scaleMultiple},
            {"scaleto", 0.1, scaleSize},
            {"scaleto", 0.2, scaleSize * scaleMultiple},
            {"scaleto", 0.1, scaleSize},
        })
        node._commonAction_guideAct = action
        node._commonAction_iniScale = scaleSize
        node:runAction(node._commonAction_guideAct)
    end
end

-- 关闭缩放引导
function M:stopScaleGuide(node)
    if node._commonAction_guideAct then
        node:stopAction(node._commonAction_guideAct)
        node._commonAction_guideAct = nil
        node:scale(node._commonAction_iniScale)
    end
end

-- 播放手指点击引导（跟随待机引导的line动作，非cycle）
function M:playFingerClickGuide()
	local wpos = self._itemNode:worldpoint()
	local anchorY = self._itemNode:getAnchorPoint().y
	wpos.y = wpos.y + self._itemNode:ch()/2 - self._itemNode:ch()*anchorY
	self._itemNode._camera._main._finger:playFingerClickGuide(wpos)
end

-- 停止手指引导(拖动/点击通用)
function M:stopFingerGuide()
	self._itemNode._camera._main._finger:closeHandGuide()
end
----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M