--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("ObjCake")
M.TAG       = "ObjCake"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 自动事件类
local AutoEvent = require("app.shapepuzzle.node.AutoEvent")

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)
	
	-- 摄像机节点
	self._camera 	= params.camera
	-- 蜡烛节点集合
	self._candleTab = {}  
	-- 烛火节点集合   
	self._fireTab 	= {}
	-- 触控锁
	self._touchLock = true
	-- 第几次咬蛋糕
	self._eatIndex  = 0
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)

	-- 图片渲染
	self:loadDisplay()
	-- 加载蛋糕实物图
	self:loadCake()
	-- 加载蜡烛
	self:loadCandle()
	-- 加载烛火
	self:loadFire()
	-- 加载自动事件
	self:loadAutoEvent()
end

-- 图片渲染
function M:loadDisplay()
	self:display("shape/puzzle/object6/item/cake/cake1.png")
end

-- 加载自动事件
function M:loadAutoEvent()
	local autoEvent = AutoEvent.new({
		itemId    = 6, 	  -- 实物ID
		delayTime = 1.5,  -- 自动事件的间隔时间
		itemNode  = self  -- 物品节点
	}):to(self):p(10000, 10000):unbindTouch()

	self._autoEvent = autoEvent
end

-- 加载蛋糕实物图
function M:loadCake()
	local cake = D.img("shape/puzzle/object6/item/cake/cake1.png"):to(self, 1010):p(self:cw()/2, self:ch()/2):hide()
	self._cake = cake
end

-- 加载蜡烛
function M:loadCandle()
	local posTab = {ccp(self:cw()/2-66.3, 317.0), ccp(self:cw()/2+0.8, 317.0), ccp(self:cw()/2+61.3, 317.0)}
	for i = 1, 3 do
		local candle = self:createCandle(posTab[i]):hide()
		table.insert(self._candleTab, candle)
	end
end

-- 加载烛火
function M:loadFire()
	local posTab = {ccp(self:cw()/2-66.3, 380), ccp(self:cw()/2+0.8, 380), ccp(self:cw()/2+61.3, 380)}
	for i = 1, 3 do
		local fire = self:createFire(posTab[i])
		table.insert(self._fireTab, fire)
	end
	self:playFireAni()
end

-- 创建单个蜡烛
function M:createCandle(pos)
	local candle = D.img("shape/puzzle/object6/item/candle.png"):to(self, 1020):p(pos):hide()
	return candle
end

-- 创建单个烛火
function M:createFire(pos)
	local fire = D.img("shape/puzzle/object6/item/fire/fire1.png"):to(self, 1030):p(pos):scale(0.8):hide()
	return fire
end

-- 显示实物图
function M:showItem()
	for i, v in pairs(self._camera._shapeTab) do
		if not tolua.isnull(v) then
			v:hide()
		end
	end
	
	self._cake:show()
	for i = 1, 3 do
		self._candleTab[i]:show()
		self._fireTab[i]:show()
	end
end

-- 播放蜡烛点燃后持续燃烧帧动画方法
function M:playFireAni()
	-- 播放音效[sfx14712024]点燃蜡烛
	soundEffect:playEffectsfx14712024()
	for i, v in pairs(self._fireTab) do
		v:show()
		v:line({
			{"imagerange", "shape/puzzle/object6/item/fire/fire", 1, 4, 0.1},
			{"fn", function ()
				v._cycleFireAct = A.cycle({
					{"imagerange", "shape/puzzle/object6/item/fire/fire", 3, 6, 0.1},
				})
				v:runAction(v._cycleFireAct)
			end}
		})
	end
end

-- 点击缩放效果
function M:playClickScale()
	self:line({
		{"scaleTo", 0.05, 0.9},
		{"scaleTo", 0.15, 1.1},
		{"scaleTo", 0.1, 1},
	})
end

-- 开始实物动作互动
function M:beganItemEffect()
	-- 播放配音[v147127]哈哈！是生日蛋糕！
	soundVoice:playEffectv147127()
	local time = sound:soundDuration("v147127")
	A.line({
		{"delay", time},
		{"fn", function ()
			self:activeAutoEvent()
		end}
	}):at(self)
end

-- 激活自动事件
function M:activeAutoEvent()
	-- 开启自动事件
	self._autoEvent:openAutoEvent()
end

-- 播放烛火被吹灭动画
function M:playFireQuitAni(event)
	for i, v in pairs(self._fireTab) do
		v:stopAllActions()
		v:line({
			{"imagerange", "shape/puzzle/object6/item/fire/fire", 8, 7, 0.1},
			{"fn", function ()
				if i == 1 then
					if event then
						event()
					end
				end
			end}
		})
	end
end

-- 播放蜡烛被拔掉动画
function M:playCandleQuitAni(event)
	self._candleTab[1]:line({
		{"fn", function ()
			-- 播放音效[sfx14712025]拔掉蜡烛
			soundEffect:playEffectsfx14712025()
		end},
		A.union({
			{"jumpBy", 0.6, ccp(-300, 300), 50, 1},
			{"rotateBy", 0.5, -180}
		}),
		{"remove"}
	})
	self._candleTab[2]:line({
		{"delay", 0.3},
		{"fn", function ()
			-- 播放音效[sfx14712025]拔掉蜡烛
			soundEffect:playEffectsfx14712025()
		end},
		A.union({
			{"jumpBy", 0.6, ccp(0, 300), 50, 1},
		}),
		{"remove"}
	})
	self._candleTab[3]:line({
		{"delay", 0.6},
		{"fn", function ()
			-- 播放音效[sfx14712025]拔掉蜡烛
			soundEffect:playEffectsfx14712025()
		end},
		A.union({
			{"jumpBy", 0.6, ccp(300, 300), 50, 1},
			{"rotateBy", 0.5, 180}
		}),
		{"fn", function ()
			if event then
				event()
			end
		end},
		{"remove"}
	})
end

-- 播放蛋糕被吃一口动画（前面4口）
function M:playEatCakeAni1(event)
	self._eatIndex = self._eatIndex + 1
	self._cake:display("shape/puzzle/object6/item/cake/cake"..(self._eatIndex+1)..".png")
	-- 播放音效[sfx14712026]吃蛋糕
	soundEffect:playEffectsfx14712026()
	if event then
		event()
	end
end

-- 播放蛋糕剩下全吃掉动画
function M:playEatCakeAni2()
	self:line({
		{"fn", function ()
			self._cake:display("shape/puzzle/object6/item/cake/cake6.png")
			-- 播放音效[sfx14712026]吃蛋糕
			soundEffect:playEffectsfx14712026()
		end},
		{"delay", 0.1},
		{"fn", function ()
			self._cake:display("shape/puzzle/object6/item/cake/cake7.png")
			-- 播放音效[sfx14712026]吃蛋糕
			soundEffect:playEffectsfx14712026()
		end},
		{"delay", 0.1},
		{"fn", function ()
			self._cake:display("shape/puzzle/object6/item/cake/cake8.png")
			-- 播放音效[sfx14712026]吃蛋糕
			soundEffect:playEffectsfx14712026()
		end},
		{"delay", 0.1},
		{"fn", function ()
			self._camera._main:clearActiveParticle()
			self._camera._main:enterNextLevel()
		end},
		{"remove"}
	})
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()
	
	self:playClickScale()
	self._autoEvent:clickEvent()
	return true
end

function M:onTouchMoved(x, y, touches)


end

function M:onTouchEnded(x, y, touches)
	
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M