--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("ObjCastle")
M.TAG       = "ObjCastle"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 自动事件类
local AutoEvent = require("app.shapepuzzle.node.AutoEvent")

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	-- 摄像机节点
	self._camera = params.camera
	-- 触控锁
	self._touchLock = true
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)

	-- 图片渲染
	self:loadDisplay()
	-- 加载城堡
	self:loadCastle()
	-- 加载国旗
	self:loadFlag()
	-- 加载女王
	self:loadQueen()
	-- 加载自动事件
	self:loadAutoEvent()
end

-- 图片渲染
function M:loadDisplay()
	self:display("shape/puzzle/object5/item/castle1.png")
end

-- 加载自动事件
function M:loadAutoEvent()
	local autoEvent = AutoEvent.new({
		itemId    = 5, 	  -- 实物ID
		delayTime = 2,	  -- 自动事件的间隔时间
		itemNode  = self  -- 物品节点
	}):to(self):p(10000, 10000):unbindTouch()

	self._autoEvent = autoEvent
end

-- 加载城堡
function M:loadCastle()
	local castle1 = D.img("shape/puzzle/object5/item/castle1.png"):to(self, 1040):p(self:cw()/2, self:ch()/2):hide()
	local castle2 = D.img("shape/puzzle/object5/item/castle2.png"):to(self, 1020):p(self:cw()/2, self:ch()/2):hide()
	self._castle1 = castle1
	self._castle2 = castle2
end

-- 加载国旗
function M:loadFlag()
	local flag = D.img("shape/puzzle/object5/item/flag/flag1.png"):to(self, 1010):p(self:cw()/2+20, 393):hide()
	self._flag = flag
end

-- 加载女王
function M:loadQueen()
	local queen = D.img("shape/puzzle/object5/item/queen/queen1.png"):to(self, 1030):p(self:cw()/2+10, 206):hide()
	self._queen= queen
end

-- 显示实物图
function M:showItem()
	for i, v in pairs(self._camera._shapeTab) do
		if not tolua.isnull(v) then
			v:hide()
		end
	end
	
	self._castle1:show()
	self._castle2:show()
end

-- 点击缩放效果
function M:playClickScale()
	self:line({
		{"scaleTo", 0.05, 0.95},
		{"scaleTo", 0.15, 1.05},
		{"scaleTo", 0.1, 1},
	})
end

-- 开始实物动作互动
function M:beganItemEffect()
	-- 播放配音[v147126]好华丽的城堡！
	soundVoice:playEffectv147126()
	
	if V.h > 640 then
		local time = sound:soundDuration("v147126")
		A.line({
			{"delay", time},
			{"fn", function ()
				self:activeAutoEvent()
			end}
		}):at(self)
	else
		self:line({
			{"moveBy", 0.5, ccp(0, -40)},
			{"delay", 0.15},
			{"fn", function ()
				local time = sound:soundDuration("v147126")
				A.line({
					{"delay", time},
					{"fn", function ()
						self:activeAutoEvent()
					end}
				}):at(self)
			end}
		})
	end
end

-- 激活自动事件
function M:activeAutoEvent()
	-- 开启自动事件
	self._autoEvent:openAutoEvent()
end

-- 播放升旗帧动画
function M:playFlagAni(event)
	-- 播放音效[sfx14707007]城堡升旗/旗帜冒出来的时候播放
	soundEffect:playEffectsfx14707007()
	self._flag:show()
	self._flag:line({
		{"image", "shape/puzzle/object5/item/flag/flag", 8, 0.1},
		{"fn", function ()
			if event then
				event()
			end
			self._flag:cycle({
				{"imagerange", "shape/puzzle/object5/item/flag/flag", 4, 5, 0.2},
			})
		end}
	})
end

-- 播放女王巡视帧动画
function M:playQueenAni(event)
	self._queen:show()
	-- 播放音效[sfx14707017]城堡-公主环绕
	soundEffect:playEffectsfx14707017()
	self._queen:line({
		{"image", "shape/puzzle/object5/item/queen/queen", 8, 0.15},
		{"fn",function ()
			if event then
				event()
			end
		end}
	})
end

-- 城堡消失效果
function M:quitEffect()
	self:line({
		{ "easing", "backIn", { "scaleTo", 0.5, 0 } },
		{"fn", function ()
			self._camera._main:clearActiveParticle()
			self._camera._main:enterNextLevel()
		end},
		{"remove"}
	})
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()
	
	self:playClickScale()
	self._autoEvent:clickEvent()
	return true
end

function M:onTouchMoved(x, y, touches)


end

function M:onTouchEnded(x, y, touches)
	
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M