--[[

Copyright (c) 2012-2021 Baby-Bus.com
Author: cxy
Date: 2021-1-13

]]
--[[!--

游戏角色类，定义游戏角色公用操作方法及逻辑实现。

-   定义游戏角色相关信息。

]]

----------------------
-- 类
----------------------
local M     = classSpriteTouch("ObjChristmasTree")
M.TAG       = "ObjChristmasTree"


----------------------
-- 公共参数
----------------------
-- [常量]
-- ..
-- 自动事件类
local AutoEvent = require("app.shapepuzzle.node.AutoEvent")

----------------------
-- 构造方法
----------------------
--[[--

构造方法，定义结点实例初始化逻辑

### Parameters:
-   table **params**    参数集合

### Return: 
-   object              对象实例

]]
function M:ctor(params)
	-- [超类调用]
	M.super.ctor(self, params)

	-- 摄像机节点
	self._camera    = params.camera
	-- 圣诞彩灯节点集合
	self._lampTab   = {}
	-- 触控锁
	self._touchLock = true
end

----------------------
-- 结点渲染 
----------------------
--[[--

实体渲染，处理实体结点加载、事件绑定等相关操作

]]
function M:onRender()
	-- [超类调用]
	M.super.onRender(self)

	-- 图片渲染
	self:loadDisplay()
	-- 加载树干实物图
	self:loadTrunk()
	-- 加载圣诞树身(三角叶群)
	self:loadBody()
	-- 加载圣诞树顶部星星
	self:loadStar()
	--  加载圣诞树灯带
	self:loadLampline()
	--  加载圣诞彩灯
	self:loadLamp()
	-- 加载自动事件
	self:loadAutoEvent()
end

-- 图片渲染
function M:loadDisplay()
	self:display("shape/puzzle/object2/item/tree.png")
end

-- 加载自动事件
function M:loadAutoEvent()
	local autoEvent = AutoEvent.new({
		itemId    = 2, 	  -- 实物ID
		delayTime = 2,	  -- 自动事件的间隔时间
		itemNode  = self  -- 物品节点
	}):to(self):p(10000, 10000):unbindTouch()

	self._autoEvent = autoEvent
end

-- 加载树干实物图
function M:loadTrunk()
	local trunk = D.img("shape/puzzle/object2/item/trunk.png"):to(self, 1010):p(self:cw()/2, 70.4):hide()
	self._trunk = trunk
end

-- 加载圣诞树身(三角叶群)
function M:loadBody()
	local body2 = D.img("shape/puzzle/object2/item/body2.png"):to(self, 1020):p(self:cw()/2, 158.4):hide()
	local body1 = D.img("shape/puzzle/object2/item/body1.png"):to(self, 1040):p(self:cw()/2, 267.4):hide()
	self._body2 = body2
	self._body1 = body1
end

-- 加载圣诞树顶部星星
function M:loadStar()
	local star = D.img("shape/puzzle/object2/item/star.png"):to(self, 1050):p(self:cw()/2, 381.9):hide()
	self._star = star
end

-- 加载圣诞树灯带
function M:loadLampline()
	local lampline1 = D.img("shape/puzzle/object2/item/lampline/lampline5.png")
	:to(self, 1060):p(self:cw()/2+17.6, 263.4):hide()
	local lampline2 = D.img("shape/puzzle/object2/item/lampline/lampline5.png")
	:to(self, 1030):p(self:cw()/2+4.6, 161.3):rotate(-7):hide()

	self._lampline1 = lampline1
	self._lampline2 = lampline2
end

-- 加载圣诞彩灯
function M:loadLamp()
	local posTab = {ccp(self:cw()/2-46.5, 253.2), ccp(self:cw()/2+11.6, 210.8), ccp(self:cw()/2+73.8, 220.6),
					ccp(self:cw()/2-57.3, 136.4), ccp(self:cw()/2+4.1, 107.2), ccp(self:cw()/2+68.0, 128.1)}
	local rotation = {10, -10, -10, 10, 10, -10}

	for i = 1, 6 do
		local lamp = self:createLamp(posTab[i], rotation[i], i)
		table.insert(self._lampTab, lamp)
	end
end

-- 创建圣诞彩灯
function M:createLamp(pos, rotation, index)
	local lamp = D.img("shape/puzzle/object2/item/lamp/lamp7.png")
	:to(self, 1070):p(pos):rotation(rotation):scale(0)

	lamp._light = D.img("shape/puzzle/object2/item/light/light"..index.."/10001.png")
	:to(lamp):p(lamp:cw()/2, lamp:ch()/2):hide()

	return lamp
end

-- 显示实物图
function M:showItem()
	for i, v in pairs(self._camera._shapeTab) do
		if not tolua.isnull(v) then
			v:hide()
		end
	end

	self._trunk:show()
	self._body2:show()
	self._body1:show()
	self._star:show()
end

-- 点击缩放效果
function M:playClickScale()
	self:line({
		{"scaleTo", 0.05, 0.9},
		{"scaleTo", 0.15, 1.1},
		{"scaleTo", 0.1, 1},
	})
end

-- 开始实物动作互动
function M:beganItemEffect()
	-- 播放配音[v147123]是漂亮的圣诞树！
	soundVoice:playEffectv147123()
	local time = sound:soundDuration("v147123")
	A.line({
		{"delay", time},
		{"fn", function ()
			self:activeAutoEvent()
		end}
	}):at(self)
end

-- 激活自动事件
function M:activeAutoEvent()
	-- 开启自动事件
	self._autoEvent:openAutoEvent()
end

-- 播放灯带出现动画
function M:playLamplineAni(event)
	-- 播放音效[sfx14707008]圣诞树/树上电线出现的时候播放
	soundEffect:playEffectsfx14707008()
	self._lampline1:show()
	self._lampline1:line({
		{"imagerange", "shape/puzzle/object2/item/lampline/lampline", 1, 5, 0.05},
		{"fn", function ()
			self._lampline2:show()
			self._lampline2:line({
				{"imagerange", "shape/puzzle/object2/item/lampline/lampline", 1, 5, 0.05},
				{"fn", function ()
					if event then
						event()
					end
				end}
			})
		end}
	})
end

-- 播放彩灯出现效果
function M:playLampGetInto()
	for i = 1, 3 do
		self._lampTab[i]:line({
			{ "easing", "backOut", { "scaleTo", 0.5, 1 } },
		})
	end

	self:line({
		{"delay", 0.25},
		{"fn", function ()
			for i = 4, 6 do
				self._lampTab[i]:line({
					{ "easing", "backOut", { "scaleTo", 0.5, 1 } },
				})
			end
		end},
		{"delay", 0.5},
		{"fn", function ()
			-- 播放音效[sfx14707016]圣诞树-彩灯闪烁
			soundEffect:playEffectsfx14707016()
			self:playRandomLightEffect()
			self:line({
				{"delay", 2}, 
				{"fn", function ()
					self:quitEffect()
				end}
			})
		end}
	})
end

-- 圣诞树消失效果
function M:quitEffect()
	self:line({
		{ "easing", "backIn", { "scaleTo", 0.5, 0 } },
		{"fn", function ()
			self._camera._main:clearActiveParticle()
			self._camera._main:enterNextLevel()
		end},
		{"remove"}
	})
end

-- 随机交错播放彩灯发光方法
function M:playRandomLightEffect()
	local tab1 = {1, 2, 3}
	local tab2 = {4, 5, 6}
	local ranTab1 = T.shuffle(tab1)
	local ranTab2 = T.shuffle(tab2)
	
	
	self:line({
		{"fn", function () self:lightTurnOn(ranTab1[1]) end},
		{"delay", 0.03*6},
		{"fn", function () self:lightTurnOn(ranTab1[2]) end},
		{"delay", 0.03*6},
		{"fn", function () self:lightTurnOn(ranTab1[3]) end},
		{"delay", 0.03*6},
		{"fn", function ()
			self:playRandomLightEffect()
		end}
	})

	self:line({
		{"fn", function () self:lightTurnOn(ranTab2[1]) end},
		{"delay", 0.03*6},
		{"fn", function () self:lightTurnOn(ranTab2[2]) end},
		{"delay", 0.03*6},
		{"fn", function () self:lightTurnOn(ranTab2[3]) end},
})
	
end

-- 彩灯发光帧动画方法
function M:lightTurnOn(index)
	local lamp = self._lampTab[index]
	lamp:display("shape/puzzle/object2/item/lamp/lamp"..index..".png")
	lamp._light:show()
	lamp._light:line({
		{"imagerange", "shape/puzzle/object2/item/light/light"..index.."/1000", 1, 6, 0.03},
		{"fn", function ()
			self:lightTurnOff(index)
		end}
	})
end

-- 彩灯熄灯效果方法
function M:lightTurnOff(index)
	local lamp = self._lampTab[index]
	lamp:display("shape/puzzle/object2/item/lamp/lamp7.png")
	lamp._light:hide()
end

----------------------
-- 结点析构
----------------------
--[[--

实体析构，处理实体结点卸载、事件解除绑定等相关操作

]]
function M:onDestructor()
	-- [超类调用]
	M.super.onDestructor(self)
end


----------------------
-- 点击
----------------------

function M:onTouchBegan(x, y, touches)
	if self._touchLock then return end
	-- 播放音效[sfx14700001]点击
	soundEffect:playEffectsfx14700001()
	
	self:playClickScale()
	self._autoEvent:clickEvent()
	return true
end

function M:onTouchMoved(x, y, touches)


end

function M:onTouchEnded(x, y, touches)
	
end


----------------------
-- 验证
----------------------
-- 验证合法性[构造函数参数]
function M:assertParameters(params)
	
end


return M